@@ -230,15 +230,24 @@ fn shortest_path(start: Vertex) -> Vec<(Edge, Vertex)> {
         }
     }
 
+    info!(
+        "Found predecessors for {} syntax nodes.",
+        predecessors.len()
+    );
     let mut current = end;
-    let mut res: Vec<(Edge, Vertex)> = vec![];
+
+    let mut route: Vec<(Edge, Vertex)> = vec![];
+    let mut cost = 0;
     while let Some(Some((node, edge))) = predecessors.remove(&current) {
-        res.push((edge, node.clone()));
+        route.push((edge, node.clone()));
+        cost += edge.cost();
+
         current = node;
     }
+    info!("Found found a path of {} with cost {}.", route.len(), cost);
 
-    res.reverse();
-    res
+    route.reverse();
+    route
 }
 
 const NOVEL_TREE_THRESHOLD: u64 = 20;
@@ -253,6 +262,7 @@ fn neighbours<'a>(v: &Vertex<'a>) -> Vec<(Edge, Vertex<'a>)> {
                 .abs() as u64;
 
             // Both nodes are equal, the happy case.
+            // TODO: this is only OK if we've not changed depth.
             res.push((
                 UnchangedNode { depth_difference },
                 Vertex {
@@ -311,19 +321,19 @@ fn neighbours<'a>(v: &Vertex<'a>) -> Vec<(Edge, Vertex<'a>)> {
         if let (
             Syntax::Atom {
                 content: lhs_content,
-                is_comment: lhs_is_comment,
+                is_comment: true,
                 ..
             },
             Syntax::Atom {
                 content: rhs_content,
-                is_comment: rhs_is_comment,
+                is_comment: true,
                 ..
             },
         ) = (lhs_syntax, rhs_syntax)
         {
             // Both sides are comments and their content is reasonably
             // similar.
-            if *lhs_is_comment && *rhs_is_comment && lhs_content != rhs_content {
+            if lhs_content != rhs_content {
                 let levenshtein_pct =
                     (normalized_levenshtein(lhs_content, rhs_content) * 100.0).round() as u8;
                 res.push((
@@ -599,7 +609,6 @@ mod tests {
             "]",
             pos_helper(2),
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_list(
             &arena,
@@ -609,7 +618,7 @@ mod tests {
             "]",
             pos_helper(2),
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -643,7 +652,6 @@ mod tests {
             "]",
             pos_helper(2),
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_list(
             &arena,
@@ -656,7 +664,7 @@ mod tests {
             "]",
             pos_helper(3),
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -694,7 +702,6 @@ mod tests {
             "]",
             pos_helper(4),
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_list(
             &arena,
@@ -707,7 +714,7 @@ mod tests {
             "}",
             pos_helper(4),
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -742,10 +749,9 @@ mod tests {
             Syntax::new_atom(&arena, col_helper(2, 0), "bar"),
             Syntax::new_atom(&arena, col_helper(2, 1), "foo"),
         ];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_atom(&arena, col_helper(1, 0), "foo")];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -780,9 +786,9 @@ mod tests {
             "]",
             pos_helper(2),
         )];
-        init_info(&lhs);
 
         let rhs = vec![];
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -817,9 +823,9 @@ mod tests {
             ),
             Syntax::new_atom(&arena, col_helper(2, 2), ";"),
         ];
-        init_info(&lhs);
 
         let rhs = vec![];
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -874,7 +880,6 @@ mod tests {
             "]",
             pos_helper(100),
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_list(
             &arena,
@@ -907,7 +912,7 @@ mod tests {
             "]",
             pos_helper(100),
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -940,14 +945,13 @@ mod tests {
             pos_helper(1),
             "the quick brown fox",
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_comment(
             &arena,
             pos_helper(1),
             "the quick brown cat",
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -975,10 +979,9 @@ mod tests {
             pos_helper(1),
             "the quick brown fox",
         )];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_comment(&arena, pos_helper(1), "foo bar")];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -1005,14 +1008,13 @@ mod tests {
             Syntax::new_comment(&arena, pos_helper(1), "the quick brown fox"),
             Syntax::new_comment(&arena, pos_helper(2), "the quick brown thing"),
         ];
-        init_info(&lhs);
 
         let rhs = vec![Syntax::new_comment(
             &arena,
             pos_helper(1),
             "the quick brown fox.",
         )];
-        init_info(&rhs);
+        init_info(&lhs, &rhs);
 
         let start = Vertex {
             lhs_syntax: lhs.get(0).copied(),
@@ -1033,4 +1035,28 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn mark_syntax_equal_atoms() {
+        let arena = Arena::new();
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo");
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "foo");
+        init_info(&[lhs], &[rhs]);
+
+        mark_syntax(Some(lhs), Some(rhs));
+        assert_eq!(lhs.info().change.get(), Some(ChangeKind::Unchanged(rhs)));
+        assert_eq!(rhs.info().change.get(), Some(ChangeKind::Unchanged(lhs)));
+    }
+
+    #[test]
+    fn mark_syntax_different_atoms() {
+        let arena = Arena::new();
+        let lhs = Syntax::new_atom(&arena, pos_helper(1), "foo");
+        let rhs = Syntax::new_atom(&arena, pos_helper(1), "bar");
+        init_info(&[lhs], &[rhs]);
+
+        mark_syntax(Some(lhs), Some(rhs));
+        assert_eq!(lhs.info().change.get(), Some(ChangeKind::Novel));
+        assert_eq!(rhs.info().change.get(), Some(ChangeKind::Novel));
+    }
 }
