@@ -414,15 +414,29 @@ impl NewlinePositions {
     }
 }
 
+/// Return the length of `s` in codepoints. This ensures that it's
+/// safe to slice `s` at this boundary.
+pub fn codepoint_len(s: &str) -> usize {
+    s.chars().count()
+}
+
+/// The first `len` codepoints of `s`. This is safer than slicing by
+/// bytes, which panics if the byte isn't on a codepoint boundary.
+pub fn substring_by_codepoint(s: &str, start: usize, end: usize) -> &str {
+    let byte_start = s.char_indices().nth(start).unwrap().0;
+    match s.char_indices().nth(end) {
+        Some(byte_end) => &s[byte_start..byte_end.0],
+        None => &s[byte_start..],
+    }
+}
+
 /// Ensure that every line in `s` has this length. Pad short lines and
 /// truncate long lines.
-pub fn enforce_length(s: &str, line_length: usize) -> String {
+pub fn enforce_exact_length(s: &str, line_length: usize) -> String {
     let mut result = String::with_capacity(s.len());
     for line in s.lines() {
-        // TODO: use length in chars not bytes.
-        if line.len() > line_length {
-            // Truncate.
-            result.push_str(&line[0..line_length]);
+        if codepoint_len(line) > line_length {
+            result.push_str(substring_by_codepoint(line, 0, line_length));
             result.push('\n');
         } else {
             // Pad with spaces.
@@ -439,8 +453,7 @@ pub fn enforce_max_length(s: &str, line_length: usize) -> String {
     for line in s.lines() {
         // TODO: use length in chars not bytes.
         if line.len() > line_length {
-            // Truncate.
-            result.push_str(&line[0..line_length]);
+            result.push_str(substring_by_codepoint(line, 0, line_length));
             result.push('\n');
         } else {
             result.push_str(&format!("{}\n", line));
@@ -467,7 +480,7 @@ mod tests {
     use pretty_assertions::assert_eq;
 
     #[test]
-    fn from_ranges_first_line() {
+    fn from_offsets_first_line() {
         let newline_positions: NewlinePositions = "foo".into();
         let line_spans = newline_positions.from_offsets(1, 3);
         assert_eq!(
@@ -481,7 +494,7 @@ mod tests {
     }
 
     #[test]
-    fn from_ranges_first_char() {
+    fn from_offsets_first_char() {
         let newline_positions: NewlinePositions = "foo".into();
         let line_spans = newline_positions.from_offsets(0, 0);
         assert_eq!(
@@ -524,7 +537,7 @@ mod tests {
     }
 
     #[test]
-    fn from_ranges_split_over_multiple_lines() {
+    fn from_offsets_split_over_multiple_lines() {
         let newline_positions: NewlinePositions = "foo\nbar\nbaz\naaaaaaaaaaa".into();
         let line_spans = newline_positions.from_offsets(5, 10);
 
@@ -547,13 +560,13 @@ mod tests {
 
     #[test]
     fn enforce_length_short() {
-        let result = enforce_length("foo\nbar\n", 5);
+        let result = enforce_exact_length("foo\nbar\n", 5);
         assert_eq!(result, "foo  \nbar  \n");
     }
 
     #[test]
     fn enforce_length_long() {
-        let result = enforce_length("foobar\nbarbaz\n", 3);
+        let result = enforce_exact_length("foobar\nbarbaz\n", 3);
         assert_eq!(result, "foo\nbar\n");
     }
 
@@ -603,4 +616,9 @@ mod tests {
         // Intervals are inclusive of `start` but exclusive of `end`.
         assert_eq!(group.lhs_lines(), vec![1.into(), 2.into()])
     }
+
+    #[test]
+    fn codepoint_len_non_ascii() {
+        assert_eq!(codepoint_len("ƒoo"), 3);
+    }
 }
