@@ -133,6 +133,7 @@ pub fn from_extension(extension: &OsStr) -> Option<Language> {
                 // Punctuation
                 Regex::new(r"[:,]").unwrap(),
                 // Double-quoted strings
+                Regex::new(r#""((\\.)|[^"])*""#).unwrap(),
             ],
             comment_patterns: vec![],
             open_delimiter_pattern: Regex::new(r"(\[|\{)").unwrap(),
@@ -190,22 +191,6 @@ pub fn from_extension(extension: &OsStr) -> Option<Language> {
             open_delimiter_pattern: Regex::new(r"(\[|\{|\()").unwrap(),
             close_delimiter_pattern: Regex::new(r"(\]|\}|\))").unwrap(),
         }),
-        "scm" => Some(Language {
-            name: "Scheme".into(),
-            atom_patterns: vec![
-                // Numbers
-                Regex::new(r"[0-9]+").unwrap(),
-                // Symbols (e.g. variable names)
-                Regex::new(r"#?[a-zA-Z0-9_?:/*=-]+").unwrap(),
-                // Operators
-                Regex::new(r"[`',#.]").unwrap(),
-                // Double-quoted strings
-                Regex::new(r#""((\\.)|[^"])*""#).unwrap(),
-            ],
-            comment_patterns: vec![Regex::new(r";.*").unwrap()],
-            open_delimiter_pattern: Regex::new(r"(\[|\{|\()").unwrap(),
-            close_delimiter_pattern: Regex::new(r"(\]|\}|\))").unwrap(),
-        }),
         _ => None,
     }
 }
@@ -695,11 +680,18 @@ pub mod tests {
             &[
                 Syntax::new_comment(
                     &arena,
-                    vec![SingleLineSpan {
-                        line: 0.into(),
-                        start_col: 0,
-                        end_col: 6,
-                    }],
+                    vec![
+                        SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 0,
+                            end_col: 6,
+                        },
+                        SingleLineSpan {
+                            line: 1.into(),
+                            start_col: 0,
+                            end_col: 0,
+                        },
+                    ],
                     "// foo\n",
                 ),
                 Syntax::new_atom(
@@ -740,34 +732,6 @@ pub mod tests {
         );
     }
 
-    #[test]
-    fn test_parse_indented_multiline_comment() {
-        let arena = Arena::new();
-
-        assert_syntaxes(
-            &parse(&arena, "  /* foo\n  bar */", &lang()),
-            // Deliberately construct an Atom directly, because
-            // Syntax::new_comment has the multiline logic.
-            &[&Atom {
-                info: crate::syntax::SyntaxInfo::new(0),
-                content: "/* foo\nbar */".into(),
-                is_comment: true,
-                position: vec![
-                    SingleLineSpan {
-                        line: 0.into(),
-                        start_col: 2,
-                        end_col: 8,
-                    },
-                    SingleLineSpan {
-                        line: 1.into(),
-                        start_col: 2,
-                        end_col: 8,
-                    },
-                ],
-            }],
-        );
-    }
-
     #[test]
     fn test_parse_list() {
         let arena = Arena::new();
