@@ -3,14 +3,12 @@
 #![allow(clippy::mutable_key_type)] // Hash for Syntax doesn't use mutable fields.
 
 use itertools::{EitherOrBoth, Itertools};
-use lazy_static::lazy_static;
-use regex::Regex;
 use std::cell::Cell;
-use std::cmp::{max, min};
+use std::cmp::min;
 use std::collections::hash_map::DefaultHasher;
 use std::collections::HashMap;
-use std::fmt;
 use std::hash::{Hash, Hasher};
+use std::{env, fmt};
 use typed_arena::Arena;
 
 use crate::lines::{LineGroup, LineNumber, NewlinePositions};
@@ -110,15 +108,18 @@ impl<'a> fmt::Debug for Syntax<'a> {
                     .field("open_position", &dbg_pos(open_position))
                     .field("children", &children)
                     .field("close_content", &close_content)
-                    .field("close_position", &dbg_pos(close_position))
-                    .field("change", &info.change.get());
+                    .field("close_position", &dbg_pos(close_position));
 
-                let next_s = match info.next.get() {
-                    Some(List { .. }) => "Some(List)",
-                    Some(Atom { .. }) => "Some(Atom)",
-                    None => "None",
-                };
-                ds.field("next", &next_s);
+                if env::var("DFT_VERBOSE").is_ok() {
+                    ds.field("change", &info.change.get());
+
+                    let next_s = match info.next.get() {
+                        Some(List { .. }) => "Some(List)",
+                        Some(Atom { .. }) => "Some(Atom)",
+                        None => "None",
+                    };
+                    ds.field("next", &next_s);
+                }
 
                 ds.finish()
             }
@@ -129,16 +130,18 @@ impl<'a> fmt::Debug for Syntax<'a> {
                 ..
             } => {
                 let mut ds = f.debug_struct(&format!("Atom id:{}", self.id()));
-                ds.field("content", &content)
-                    .field("change", &info.change.get());
+                ds.field("content", &content);
                 ds.field("position", &dbg_pos(position));
 
-                let next_s = match info.next.get() {
-                    Some(List { .. }) => "Some(List)",
-                    Some(Atom { .. }) => "Some(Atom)",
-                    None => "None",
-                };
-                ds.field("next", &next_s);
+                if env::var("DFT_VERBOSE").is_ok() {
+                    ds.field("change", &info.change.get());
+                    let next_s = match info.next.get() {
+                        Some(List { .. }) => "Some(List)",
+                        Some(Atom { .. }) => "Some(Atom)",
+                        None => "None",
+                    };
+                    ds.field("next", &next_s);
+                }
 
                 ds.finish()
             }
@@ -146,21 +149,6 @@ impl<'a> fmt::Debug for Syntax<'a> {
     }
 }
 
-fn trim_left(max_trim: usize, content: &str, pos: SingleLineSpan) -> (String, SingleLineSpan) {
-    let chars: Vec<_> = content.chars().collect();
-
-    match chars.iter().position(|c| *c != ' ' && *c != '\t') {
-        Some(first_non_whitespace) => {
-            let skip_num = max(max_trim, first_non_whitespace);
-
-            let mut new_pos = pos;
-            new_pos.start_col += skip_num;
-            (chars.iter().skip(skip_num).collect(), new_pos)
-        }
-        None => (content.to_string(), pos),
-    }
-}
-
 impl<'a> Syntax<'a> {
     pub fn new_list(
         arena: &'a Arena<Syntax<'a>>,
@@ -214,27 +202,7 @@ impl<'a> Syntax<'a> {
         position: Vec<SingleLineSpan>,
         content: &str,
     ) -> &'a Syntax<'a> {
-        // Ignore leading whitespace in multiline comments, so changes
-        // in comment indentation are ignored.
-        let first_line_indent = match position.first() {
-            Some(line_pos) => line_pos.start_col,
-            None => 0,
-        };
-
-        let mut new_lines: Vec<String> = vec![];
-        let mut new_position = vec![];
-        for (i, (line, span)) in content.lines().zip(position).enumerate() {
-            if i == 0 {
-                new_lines.push(line.to_string());
-                new_position.push(span);
-            } else {
-                let (new_line, new_span) = trim_left(first_line_indent, line, span);
-                new_lines.push(new_line);
-                new_position.push(new_span);
-            }
-        }
-
-        Self::new_atom_(arena, new_position, &new_lines.join("\n"), true)
+        Self::new_atom_(arena, position, content, true)
     }
 
     #[allow(clippy::mut_from_ref)] // Clippy doesn't understand arenas.
@@ -354,7 +322,22 @@ impl<'a> Syntax<'a> {
                     is_comment: rhs_is_comment,
                     ..
                 },
-            ) => lhs_content == rhs_content && lhs_is_comment == rhs_is_comment,
+            ) => {
+                if lhs_is_comment != rhs_is_comment {
+                    return false;
+                }
+
+                if *lhs_is_comment {
+                    let is_multiline = lhs_content.lines().count() > 1;
+
+                    if is_multiline {
+                        let lhs_lines = lhs_content.lines().map(|l| l.trim_start()).collect_vec();
+                        let rhs_lines = rhs_content.lines().map(|l| l.trim_start()).collect_vec();
+                        return lhs_lines == rhs_lines;
+                    }
+                }
+                lhs_content == rhs_content
+            }
             (
                 List {
                     open_content: lhs_open_content,
@@ -439,11 +422,20 @@ impl<'a> Syntax<'a> {
     }
 }
 
-pub fn init_info<'a>(roots: &[&'a Syntax<'a>]) {
-    set_unique_id(roots, 0);
+pub fn init_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'a>]) {
+    let next_id = init_info_single(lhs_roots, 0);
+    init_info_single(rhs_roots, next_id);
+}
+
+/// Initialise all the fields in SyntaxInfo.
+///
+/// Return the next unique ID available, so we can ensure LHS and RHS
+/// have different IDs.
+pub fn init_info_single<'a>(roots: &[&'a Syntax<'a>], first_id: u64) -> u64 {
     set_next(roots, None);
     set_prev(roots, None);
     set_num_ancestors(roots, 0);
+    set_unique_id(roots, first_id)
 }
 
 fn set_unique_id<'a>(nodes: &[&'a Syntax<'a>], prev_id: u64) -> u64 {
@@ -531,7 +523,7 @@ impl MatchKind {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct MatchedPos {
     pub kind: MatchKind,
     pub pos: Vec<SingleLineSpan>,
@@ -551,11 +543,12 @@ fn split_comment_words(
 
     // TODO: merge adjacent single-line comments unless there are
     // blank lines between them.
-    lazy_static! {
-        static ref WORD_BOUNDARY_RE: Regex = Regex::new(r"\b").unwrap();
-    }
-    let content_parts: Vec<_> = WORD_BOUNDARY_RE.split(content).collect();
-    let other_parts: Vec<_> = WORD_BOUNDARY_RE.split(opposite_content).collect();
+    let content_parts: Vec<_> = content
+        .split_inclusive(&[' ', '\n', '\t'] as &[char])
+        .collect();
+    let other_parts: Vec<_> = opposite_content
+        .split_inclusive(&[' ', '\n', '\t'] as &[char])
+        .collect();
 
     let content_newlines = NewlinePositions::from(content);
     let opposite_content_newlines = NewlinePositions::from(opposite_content);
@@ -1120,6 +1113,22 @@ mod tests {
         assert_ne!(comment, atom);
     }
 
+    #[test]
+    fn test_multiline_comment_ignores_leading_whitespace() {
+        let pos = vec![SingleLineSpan {
+            line: 0.into(),
+            start_col: 2,
+            end_col: 3,
+        }];
+
+        let arena = Arena::new();
+
+        let x = Syntax::new_comment(&arena, pos.clone(), "foo\nbar");
+        let y = Syntax::new_comment(&arena, pos, "foo\n    bar");
+
+        assert_eq!(x, y);
+    }
+
     #[test]
     fn test_atom_equality_ignores_change() {
         assert_eq!(
@@ -1152,4 +1161,35 @@ mod tests {
             }
         );
     }
+
+    #[test]
+    fn test_split_comment_words_basic() {
+        let content = "abc";
+        let pos = vec![SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 3,
+        }];
+
+        let opposite_content = "def";
+        let opposite_pos = vec![SingleLineSpan {
+            line: 0.into(),
+            start_col: 0,
+            end_col: 3,
+        }];
+
+        let res = split_comment_words(content, &pos, opposite_content, &opposite_pos, &[]);
+        assert_eq!(
+            res,
+            vec![MatchedPos {
+                kind: MatchKind::ChangedCommentPart,
+                pos: vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 3
+                }],
+                prev_opposite_pos: vec![]
+            },]
+        );
+    }
 }
